"""Constants for the kmtronic integration."""

DOMAIN = "kmtronic"

CONF_REVERSE = "reverse"

DATA_HUB = "hub"
DATA_COORDINATOR = "coordinator"

MANUFACTURER = "KMtronic"
ATTR_MANUFACTURER = "manufacturer"
ATTR_IDENTIFIERS = "identifiers"

UPDATE_LISTENER = "update_listener"
