"""Constants for the sma integration."""

DOMAIN = "sma"

PYSMA_COORDINATOR = "coordinator"
PYSMA_OBJECT = "pysma"
PYSMA_REMOVE_LISTENER = "remove_listener"
PYSMA_SENSORS = "pysma_sensors"

PLATFORMS = ["sensor"]

CONF_CUSTOM = "custom"
CONF_FACTOR = "factor"
CONF_GROUP = "group"
CONF_KEY = "key"
CONF_UNIT = "unit"
DEVICE_INFO = "device_info"

DEFAULT_SCAN_INTERVAL = 5

GROUPS = ["user", "installer"]
