"""Automatically generated by hassfest.

To update, run python3 -m script.hassfest
"""

# fmt: off

DHCP = [
    {
        "domain": "august",
        "hostname": "connect",
        "macaddress": "D86162*"
    },
    {
        "domain": "august",
        "hostname": "connect",
        "macaddress": "B8B7F1*"
    },
    {
        "domain": "august",
        "hostname": "august*",
        "macaddress": "E076D0*"
    },
    {
        "domain": "axis",
        "hostname": "axis-00408c*",
        "macaddress": "00408C*"
    },
    {
        "domain": "axis",
        "hostname": "axis-accc8e*",
        "macaddress": "ACCC8E*"
    },
    {
        "domain": "axis",
        "hostname": "axis-b8a44f*",
        "macaddress": "B8A44F*"
    },
    {
        "domain": "blink",
        "hostname": "blink*",
        "macaddress": "B85F98*"
    },
    {
        "domain": "broadlink",
        "macaddress": "34EA34*"
    },
    {
        "domain": "broadlink",
        "macaddress": "24DFA7*"
    },
    {
        "domain": "broadlink",
        "macaddress": "A043B0*"
    },
    {
        "domain": "broadlink",
        "macaddress": "B4430D*"
    },
    {
        "domain": "emonitor",
        "hostname": "emonitor*",
        "macaddress": "0090C2*"
    },
    {
        "domain": "flume",
        "hostname": "flume-gw-*"
    },
    {
        "domain": "goalzero",
        "hostname": "yeti*"
    },
    {
        "domain": "gogogate2",
        "hostname": "ismartgate*"
    },
    {
        "domain": "guardian",
        "hostname": "gvc*",
        "macaddress": "30AEA4*"
    },
    {
        "domain": "guardian",
        "hostname": "guardian*",
        "macaddress": "30AEA4*"
    },
    {
        "domain": "hunterdouglas_powerview",
        "hostname": "hunter*",
        "macaddress": "002674*"
    },
    {
        "domain": "isy994",
        "hostname": "isy*",
        "macaddress": "0021B9*"
    },
    {
        "domain": "lyric",
        "hostname": "lyric-*",
        "macaddress": "48A2E6*"
    },
    {
        "domain": "lyric",
        "hostname": "lyric-*",
        "macaddress": "B82CA0*"
    },
    {
        "domain": "lyric",
        "hostname": "lyric-*",
        "macaddress": "00D02D*"
    },
    {
        "domain": "myq",
        "macaddress": "645299*"
    },
    {
        "domain": "nest",
        "macaddress": "18B430*"
    },
    {
        "domain": "nexia",
        "hostname": "xl857-*",
        "macaddress": "000231*"
    },
    {
        "domain": "nuheat",
        "hostname": "nuheat",
        "macaddress": "002338*"
    },
    {
        "domain": "nuki",
        "hostname": "nuki_bridge_*"
    },
    {
        "domain": "powerwall",
        "hostname": "1118431-*",
        "macaddress": "88DA1A*"
    },
    {
        "domain": "powerwall",
        "hostname": "1118431-*",
        "macaddress": "000145*"
    },
    {
        "domain": "rachio",
        "hostname": "rachio-*",
        "macaddress": "009D6B*"
    },
    {
        "domain": "rachio",
        "hostname": "rachio-*",
        "macaddress": "F0038C*"
    },
    {
        "domain": "rachio",
        "hostname": "rachio-*",
        "macaddress": "74C63B*"
    },
    {
        "domain": "ring",
        "hostname": "ring*",
        "macaddress": "0CAE7D*"
    },
    {
        "domain": "roomba",
        "hostname": "irobot-*",
        "macaddress": "501479*"
    },
    {
        "domain": "roomba",
        "hostname": "roomba-*",
        "macaddress": "80A589*"
    },
    {
        "domain": "samsungtv",
        "hostname": "tizen*"
    },
    {
        "domain": "screenlogic",
        "hostname": "pentair: *",
        "macaddress": "00C033*"
    },
    {
        "domain": "sense",
        "hostname": "sense-*",
        "macaddress": "009D6B*"
    },
    {
        "domain": "sense",
        "hostname": "sense-*",
        "macaddress": "DCEFCA*"
    },
    {
        "domain": "smartthings",
        "hostname": "st*",
        "macaddress": "24FD5B*"
    },
    {
        "domain": "smartthings",
        "hostname": "smartthings*",
        "macaddress": "24FD5B*"
    },
    {
        "domain": "smartthings",
        "hostname": "hub*",
        "macaddress": "24FD5B*"
    },
    {
        "domain": "smartthings",
        "hostname": "hub*",
        "macaddress": "D052A8*"
    },
    {
        "domain": "smartthings",
        "hostname": "hub*",
        "macaddress": "286D97*"
    },
    {
        "domain": "solaredge",
        "hostname": "target",
        "macaddress": "002702*"
    },
    {
        "domain": "somfy_mylink",
        "hostname": "somfy_*",
        "macaddress": "B8B7F1*"
    },
    {
        "domain": "squeezebox",
        "hostname": "squeezebox*",
        "macaddress": "000420*"
    },
    {
        "domain": "tado",
        "hostname": "tado*"
    },
    {
        "domain": "tesla",
        "hostname": "tesla_*",
        "macaddress": "4CFCAA*"
    },
    {
        "domain": "tesla",
        "hostname": "tesla_*",
        "macaddress": "044EAF*"
    },
    {
        "domain": "tesla",
        "hostname": "tesla_*",
        "macaddress": "98ED5C*"
    },
    {
        "domain": "toon",
        "hostname": "eneco-*",
        "macaddress": "74C63B*"
    },
    {
        "domain": "tplink",
        "hostname": "hs*",
        "macaddress": "1C3BF3*"
    },
    {
        "domain": "tplink",
        "hostname": "hs*",
        "macaddress": "50C7BF*"
    },
    {
        "domain": "tplink",
        "hostname": "hs*",
        "macaddress": "68FF7B*"
    },
    {
        "domain": "tplink",
        "hostname": "hs*",
        "macaddress": "98DAC4*"
    },
    {
        "domain": "tplink",
        "hostname": "hs*",
        "macaddress": "B09575*"
    },
    {
        "domain": "tplink",
        "hostname": "k[lp]*",
        "macaddress": "1C3BF3*"
    },
    {
        "domain": "tplink",
        "hostname": "k[lp]*",
        "macaddress": "50C7BF*"
    },
    {
        "domain": "tplink",
        "hostname": "k[lp]*",
        "macaddress": "68FF7B*"
    },
    {
        "domain": "tplink",
        "hostname": "k[lp]*",
        "macaddress": "98DAC4*"
    },
    {
        "domain": "tplink",
        "hostname": "k[lp]*",
        "macaddress": "B09575*"
    },
    {
        "domain": "tplink",
        "hostname": "lb*",
        "macaddress": "1C3BF3*"
    },
    {
        "domain": "tplink",
        "hostname": "lb*",
        "macaddress": "50C7BF*"
    },
    {
        "domain": "tplink",
        "hostname": "lb*",
        "macaddress": "68FF7B*"
    },
    {
        "domain": "tplink",
        "hostname": "lb*",
        "macaddress": "98DAC4*"
    },
    {
        "domain": "tplink",
        "hostname": "lb*",
        "macaddress": "B09575*"
    },
    {
        "domain": "tuya",
        "macaddress": "508A06*"
    },
    {
        "domain": "tuya",
        "macaddress": "7CF666*"
    },
    {
        "domain": "tuya",
        "macaddress": "10D561*"
    },
    {
        "domain": "tuya",
        "macaddress": "D4A651*"
    },
    {
        "domain": "tuya",
        "macaddress": "68572D*"
    },
    {
        "domain": "tuya",
        "macaddress": "1869D8*"
    },
    {
        "domain": "verisure",
        "macaddress": "0023C1*"
    },
    {
        "domain": "yeelight",
        "hostname": "yeelink-*"
    }
]
