"""Constants for the Elgato Light integration."""

# Integration domain
DOMAIN = "elgato"

# Home Assistant data keys
DATA_ELGATO_CLIENT = "elgato_client"

# Attributes
ATTR_ON = "on"

CONF_SERIAL_NUMBER = "serial_number"

# Services
SERVICE_IDENTIFY = "identify"
