"""Constants for the Rituals Perfume Genie integration."""
DOMAIN = "rituals_perfume_genie"

COORDINATORS = "coordinators"
DEVICES = "devices"

ACCOUNT_HASH = "account_hash"
ATTRIBUTES = "attributes"
HUBLOT = "hublot"
ID = "id"
SENSORS = "sensors"
